<?php

namespace App\Http\Controllers;

use App\Models\FolderDirectoryModel;
use App\Models\FileDocumentModel;
use App\Models\UserDirPermissionModel;
use App\Service\HelperFunctionService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Auth;
use App\Service\AuditLogService;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use App\Service\MultiService;


class FolderDirectory extends Controller
{
    const CODE = 'FD' ;
    public function createFolder(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if(!$user_details->role_id){
                throw new \Exception('Permission denied');
            }

            $input = $request->all();

            $validation = Validator::make($input,[
                'folder_name'=>"required|string|max:100|regex:/^[a-zA-Z0-9 -]*$/", // alpha numeric with space and hypen are allowed , // ^[a-zA-Z0-9 -]*$
                'parent_folder_id'=>"nullable",
                'section_code'=>"required|string|max:10",
                'folder_id_for_edit'=>"nullable|numeric",
                'is_edit_folder_name'=>"nullable",
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $validated = $validation->validated();

            if($input['is_edit_folder_name'] ){

                if( $input['folder_id_for_edit'] == 0 ){
                    throw new \Exception('Folder id is required') ;
                }

                FolderDirectoryModel::where('id', $input['folder_id_for_edit'] )->update([
                    'folder_name' => $validated['folder_name'] ,
                    'updated_by' => $user_details->id
                ]);

                AuditLogService::addAuditLog([
                    'folder_id' => $validated['parent_folder_id'] ?? 0  ,
                    'user_id' => $user_details->id,
                    'action' => 'Folder renamed to  ' . $validated['folder_name'],
                    'bussiness_id' => $user_details->bussiness_id,
                    'section_code' =>  $validated['section_code'] ,
                    'entity_type' => 2,
                    'log_type' => 2 ,
                    'text' => 'folder rename' ,
                ]) ;

                return response([ 'status' => true ,
                    'message' => 'Folder Updated successfully.'
                ]);
            }

            FolderDirectoryModel::create([
                'folder_name' => $validated['folder_name'] ,
                'section_id' => $validated['section_code'] ,
                'bussiness_id' => $user_details->bussiness_id,
                'parent_folder_id' => $validated['parent_folder_id'] ?? 0 ,
                'added_by' => $user_details->id
            ]);


            AuditLogService::addAuditLog([
                'folder_id' => $validated['parent_folder_id'] ?? 0 ,
                'user_id' =>  $user_details->id,
                'action' => $validated['folder_name'] . ' is added as a new folder' ,
                'bussiness_id' => $user_details->bussiness_id,
                'entity_type' => 2,
                'section_code' =>  $validated['section_code'] ,
                'log_type' => 1 ,
                'text' => 'folder added' ,
            ]) ;


            return response([ 'status' => true ,
                'message' => 'Folder Created successfully.'
            ]);

        }catch(\Throwable $e) {

            $err = [
                'code' => self::CODE.'03',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=>'false',
                'message'=> $e->getMessage()
            ],200);
        }

    }



    public function delFolder(  $folder_id ){

        try{

            $user_details = HelperFunctionService::doValidateForUser() ;

            $req = [
                'folder_id' => $folder_id
            ];

            $validation = Validator::make($req,[
                'folder_id'=>'required|numeric' # uniq id
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $validated = $validation->validated();

            $bsns_id = $user_details->bussiness_id ;

            //---

            FolderDirectoryModel::where('id', $validated['folder_id'] )->where('bussiness_id', $bsns_id )->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            UserDirPermissionModel::where('folder_id', $validated['folder_id'] )->where('bussiness_id', $bsns_id )->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            $folder_data = FolderDirectoryModel::select('section_id','folder_name')->where('id', $folder_id )->first() ;

            AuditLogService::addAuditLog([
                // 'file_uid' =>  $uniq_id,
                'action' => $folder_data->folder_name.' <b>folder deleted</b>' ,
                'folder_id' =>  $folder_id ,
                'section_code' => $folder_data->section_id ,
                'bussiness_id' => $bsns_id ,
                'user_id' =>  $user_details->id ,
                'entity_type' => 2,
                'log_type' => 15 ,
                'text' => 'folder del' ,
            ]) ;

            $obj = new MultiService() ;
            $obj->delRecursive(['folder_id'=> $folder_id, 'bussiness_id'=> $bsns_id ]);

            return response()->json([
                'status'=> true,
                'message'=> 'Folder deleted successfully.'
            ],200);


        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'08',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=> false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function moveEntity(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $validation = Validator::make($request->all(), [
                'target_obj' => 'required|array',
                'source_obj' => 'required|array',
                'section_code' => 'required|string',
                'current_dir_id' => 'required|numeric'
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            $validated = $validation->validated();

            if(!$validated['target_obj']['is_folder'] ){
                throw new \Exception('Can not move in document');
            }

            $parent_id = $validated['target_obj']['id'] ;

            $source_is_folder = $validated['source_obj']['is_folder'] ;

            $upd_status =  null ;
            if($source_is_folder){

                $upd_status = FolderDirectoryModel::where('id',$validated['source_obj']['id'] )->update([
                    'parent_folder_id' => $parent_id
                ]);

            }else{

                $upd_status = FileDocumentModel::where('unique_id',$validated['source_obj']['id'] )->update([
                    'folder_id' => $parent_id
                ]);
            }

            $str = $source_is_folder ? 'Folder ' : 'Document ' ;

            $msg =  $str.'Moved successfully.' ;

            if(!$upd_status){
                $msg = 'Unable to move.' ;
            }

            return response()->json([
                'status'=> true,
                'message'=> $msg
            ],200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '05',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }

    }

    // R&D going on
    public function downloadFolder($folder_id){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $get_uploaded_files = FileDocumentModel::select('path')
            ->whereNull('deleted_at')
            ->where('bussiness_id', $user_details->bussiness_id)
            ->where('folder_id', $folder_id)
            ->get()->toArray();

            echo '<pre>'; print_r($get_uploaded_files) ;

            $file_path = [] ;
            foreach($get_uploaded_files as $val){
                $path =  $val['path'] ;
                if(strlen($path) > 4 ){
                    $file_path[] = $path ;
                }
            }

            echo '<pre>'; print_r($file_path) ;

            // file copy start
            // $sourcePath = 'path/to/source/file.jpg';
            // /home/bharat/Documents/_work/nandik/filefort/filefort-backend/public/storage/upload_box
            // /home/bharat/Documents/_work/nandik/filefort/filefort-backend/storage/app/public/upload_box

            $sourcePath = '/home/bharat/Documents/_work/nandik/filefort/filefort-backend/storage/app/public/upload_box/'. 'upload_box/1703681481_10097706.txt';
            $sourcePath = '/home/bharat/Documents/_work/nandik/filefort/filefort-backend/storage/app/public/upload_box/1696185361_23941702.php';

            // Destination file path
            $destinationPath = 'path/to/destination/file_copy.jpg';
            $destinationPath = '/home/bharat/Documents/_work/nandik/filefort/filefort-backend/storage/app/public/a1/';

            // Copy the file

            // $sourcePath = 'public/upload_box/1696185361_23941702.php' ; // working

            echo $sourcePath ; echo '<br>' ;
            // if (Storage::exists($sourcePath)) {
            //     echo ' it iss ' ;
            // }else{
            //     echo ' Noo ' ;
            // }

            // full path not working ;

            //   $filePath = $request->file('file')->storeAs('upload_box', $fileName, 'public');
            // Storage::copy($sourcePath, 'public/a3/y7y.php'); // inside storage folder
            File::copy($sourcePath, '/home/bharat/Documents/_work/nandik/filefort/filefort-backend/storage/app/public/a1/5r4r.php'); // inside storage folder
             // wil copy in : /home/bharat/Documents/_work/nandik/filefort/filefort-backend/storage/app/public/a3/y7y.php
            echo ' copied 33 ';
            exit ;
            // file copy end


            $name_of_fol = 'fol__'.$folder_id ;

            $sourcePath = 'path/to/source/file.jpg';

            // Destination file path
            $destinationPath = 'path/to/destination/file_copy.jpg';

            // Copy the file
            Storage::copy($sourcePath, $destinationPath);

            echo $folder_id; exit ;

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '08',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }

    }

}
