<?php

namespace App\Http\Controllers;

use App\Models\FolderDirectoryModel;
use App\Models\FileDocumentModel;
use App\Models\UserDirPermissionModel;
use App\Service\HelperFunctionService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Auth;
use App\Service\AuditLogService;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use App\Service\MultiService;
use App\Service\RetentionService;
use ZipArchive;


class FolderDirectory extends Controller
{

    protected $retentionService;

    public function __construct(RetentionService $retentionService)
    {
        $this->retentionService = $retentionService;
    }

    const CODE = 'FD' ;
    public function createFolder(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if(!$user_details->role_id){
                throw new \Exception('Permission denied');
            }

            $input = $request->all();

            $validation = Validator::make($input,[
                'folder_name'=>"required|string|max:100|regex:/^[a-zA-Z0-9 -]*$/", // alpha numeric with space and hypen are allowed , // ^[a-zA-Z0-9 -]*$
                'parent_folder_id'=>"nullable",
                'section_code'=>"required|string|max:10",
                'folder_id_for_edit'=>"nullable|numeric",
                'is_edit_folder_name'=>"nullable",
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $validated = $validation->validated();

            if($input['is_edit_folder_name'] ){

                if( $input['folder_id_for_edit'] == 0 ){
                    throw new \Exception('Folder id is required') ;
                }

                FolderDirectoryModel::where('id', $input['folder_id_for_edit'] )->update([
                    'folder_name' => $validated['folder_name'] ,
                    'updated_by' => $user_details->id
                ]);

                AuditLogService::addAuditLog([
                    'folder_id' => $validated['parent_folder_id'] ?? 0  ,
                    'user_id' => $user_details->id,
                    'action' => 'Folder renamed to  ' . $validated['folder_name'],
                    'bussiness_id' => $user_details->bussiness_id,
                    'section_code' =>  $validated['section_code'] ,
                    'entity_type' => 2,
                    'log_type' => 2 ,
                    'text' => 'folder rename' ,
                ]) ;

                return response([ 'status' => true ,
                    'message' => 'Folder Updated successfully.'
                ]);
            }

            FolderDirectoryModel::create([
                'folder_name' => $validated['folder_name'] ,
                'section_id' => $validated['section_code'] ,
                'bussiness_id' => $user_details->bussiness_id,
                'parent_folder_id' => $validated['parent_folder_id'] ?? 0 ,
                'added_by' => $user_details->id
            ]);


            AuditLogService::addAuditLog([
                'folder_id' => $validated['parent_folder_id'] ?? 0 ,
                'user_id' =>  $user_details->id,
                'action' => $validated['folder_name'] . ' is added as a new folder' ,
                'bussiness_id' => $user_details->bussiness_id,
                'entity_type' => 2,
                'section_code' =>  $validated['section_code'] ,
                'log_type' => 1 ,
                'text' => 'folder added' ,
            ]) ;


            return response([ 'status' => true ,
                'message' => 'Folder Created successfully.'
            ]);

        }catch(\Throwable $e) {

            $err = [
                'code' => self::CODE.'03',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=>'false',
                'message'=> $e->getMessage()
            ],200);
        }

    }



    public function delFolder(  $folder_id ){

        try{

            $user_details = HelperFunctionService::doValidateForUser() ;

            $req = [
                'folder_id' => $folder_id
            ];

            $validation = Validator::make($req,[
                'folder_id'=>'required|numeric' # uniq id
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $validated = $validation->validated();

            $bsns_id = $user_details->bussiness_id ;

            //---

            FolderDirectoryModel::where('id', $validated['folder_id'] )->where('bussiness_id', $bsns_id )->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            UserDirPermissionModel::where('folder_id', $validated['folder_id'] )->where('bussiness_id', $bsns_id )->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            $folder_data = FolderDirectoryModel::select('section_id','folder_name')->where('id', $folder_id )->first() ;

            AuditLogService::addAuditLog([
                // 'file_uid' =>  $uniq_id,
                'action' => $folder_data->folder_name.' <b>folder deleted</b>' ,
                'folder_id' =>  $folder_id ,
                'section_code' => $folder_data->section_id ,
                'bussiness_id' => $bsns_id ,
                'user_id' =>  $user_details->id ,
                'entity_type' => 2,
                'log_type' => 15 ,
                'text' => 'folder del' ,
            ]) ;

            $obj = new MultiService() ;
            $obj->delRecursive(['folder_id'=> $folder_id, 'bussiness_id'=> $bsns_id ]);

            return response()->json([
                'status'=> true,
                'message'=> 'Folder deleted successfully.'
            ],200);


        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'08',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=> false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function moveEntity(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $validation = Validator::make($request->all(), [
                'target_obj' => 'required|array',
                'source_obj' => 'required|array',
                'section_code' => 'required|string',
                'current_dir_id' => 'required|numeric'
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            $validated = $validation->validated();

            if(!$validated['target_obj']['is_folder'] ){
                throw new \Exception('Can not move in document');
            }

            $parent_id = $validated['target_obj']['id'] ;

            $source_is_folder = $validated['source_obj']['is_folder'] ;

            $upd_status =  null ;
            if($source_is_folder){

                $upd_status = FolderDirectoryModel::where('id',$validated['source_obj']['id'] )->update([
                    'parent_folder_id' => $parent_id
                ]);

            }else{

                $upd_status = FileDocumentModel::where('unique_id',$validated['source_obj']['id'] )->update([
                    'folder_id' => $parent_id
                ]);
            }

            $str = $source_is_folder ? 'Folder ' : 'Document ' ;

            $msg =  $str.'Moved successfully.' ;

            if(!$upd_status){
                $msg = 'Unable to move.' ;
            }

            return response()->json([
                'status'=> true,
                'message'=> $msg
            ],200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '05',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }

    }

    public function getFolderRetention($folder_id){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $bsns_id = $user_details->bussiness_id ;

            if(!$folder_id){
                throw new \Exception('Folder id is not valid') ;
            }

            $folder_data = FolderDirectoryModel::select('retention_date', 'retention_type', 'retention_period')
                ->where('id', $folder_id )
                ->where('bussiness_id', $bsns_id)
                ->first() ;

            if(!$folder_data){
                throw new \Exception('Folder id is not valid') ;
            }

            $det_ar = $folder_data->toArray() ;

            if($det_ar['retention_type']==2){
                $ar_exp = explode(',', $det_ar->retention_period ) ;
                $det_ar['r_days'] = $ar_exp[0] ;
                $det_ar['r_month'] = $ar_exp[1] ;
                $det_ar['r_yr'] = $ar_exp[2] ;
            }

            return response()->json([
                'status'=> true,
                'data' => $det_ar
            ],200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '09',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }

    }

    public function setFolderRetention(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $req = $request->all();

            $validation = Validator::make($req, [
                'folder_id' => "required|numeric|min:1",
                'r_month' => "required|numeric",
                'r_days' => "required|numeric",
                'r_yr' => "required|numeric",
                'retention_type' => "required|numeric",
                'retention_date' => "nullable",
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            $validated = $validation->validated();         

            $retention_type = $validated['retention_type'] ;
            $ret_date = '' ;
            $retention_period = '' ;

            if($validated['retention_type']!=3){

                if($validated['retention_type']==1){
                    if($validated['retention_date'] == ''){
                        throw new \Exception('Retention date is invalid.') ;
                    }
                    $ret_date = $validated['retention_date'] ;
                }

                if($validated['retention_type']==2){
                    $retention_period = $validated['r_days'].','.$validated['r_month'].','.$validated['r_yr'] ;

                    $ret_date = date('Y-m-d') ;
                    if(isset($validated['r_days'] )){
                        $ret_date = date('Y-m-d', strtotime('+ '.$validated['r_days'].' days', strtotime($ret_date))) ;
                    }
                    if(isset($validated['r_month'] )){
                        $ret_date = date('Y-m-d', strtotime('+ '.$validated['r_month'].' months', strtotime($ret_date))) ;
                    }
                    if(isset($validated['r_yr'] )){
                        $ret_date = date('Y-m-d', strtotime('+ '.$validated['r_yr'].' years', strtotime($ret_date))) ;
                    }
                }
            }

            $bsns_id = $user_details->bussiness_id ;

            $upd_ar = [
                'updated_by' => $user_details->id,
                'retention_period' => $retention_period,
                'retention_date' => $ret_date,
                'retention_type' => $retention_type,
            ] ;


            //new code

            $doc_ret = $this->retentionService->addRetention($validated['folder_id'], $ret_date, $retention_type, $retention_period);

            // new code end


            $is_upd = FolderDirectoryModel::where('id', $validated['folder_id'])
                ->where('bussiness_id', $bsns_id)
                ->update($upd_ar);

            if($is_upd){
                $folder_id = $validated['folder_id'] ;

                $folder_data = FolderDirectoryModel::select('section_id','folder_name')->where('id', $folder_id )->first() ;

                AuditLogService::addAuditLog([
                    'action' => ' <b>Retention Updated</b>' ,
                    'folder_id' =>  $folder_id ,
                    'section_code' => $folder_data->section_id ,
                    'bussiness_id' => $bsns_id ,
                    'user_id' =>  $user_details->id ,
                    'entity_type' => 2,
                    'log_type' => 24 ,
                    'text' => 'folder retention updated' ,
                ]) ;

                
            }

            return response()->json([
                'status'=> true,
                'message'=> 'Folder Retention Updated.'
            ],200);


        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '09',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }

    }

      // R&D done
      public function downloadFolder($folder_id){ // 168

        try{
            $user_details = HelperFunctionService::doValidateForUser();

            $bsns_id = $user_details->bussiness_id ;

            if(!env("FULL_PATH_FOLDER_DOWNLOAD")){
                throw new \Exception("ER0011 :: Download path not provided");
            } 

            $destination_pre_path = env("FULL_PATH_FOLDER_DOWNLOAD");

            (new MultiService())->downloadFolderRecursive( 
                [  
                    'folder_id' => $folder_id, 
                    'bussiness_id' => $bsns_id,
                    'destination_pre_path' => $destination_pre_path
                ]);

            // delete folder after download

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '08',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }

    }


    public function zipFolder($folderPath)
    {
        $zipFileName = 'archive.zip';

        // Create a ZipArchive instance
        $zip = new ZipArchive;

        // Open the zip file
        if ($zip->open($zipFileName, ZipArchive::CREATE) === true) {

            // Add all files and directories from the given folder
            $files = new RecursiveIteratorIterator(
                new RecursiveDirectoryIterator($folderPath),
                RecursiveIteratorIterator::LEAVES_ONLY
            );

            foreach ($files as $name => $file) {
                // Skip directories (they would be added automatically)
                if (!$file->isDir()) {
                    // Get real and relative path for current file
                    $filePath = $file->getRealPath();
                    $relativePath = substr($filePath, strlen($folderPath) + 1);

                    // Add current file to archive
                    $zip->addFile($filePath, $relativePath);
                }
            }

            // Close and save the archive
            $zip->close();

            // Move the created zip file to a desired location (optional)
            File::move($zipFileName, storage_path('app/public/' . $zipFileName));

            return response()->download(storage_path('app/public/' . $zipFileName))
                ->deleteFileAfterSend(true);
        }

        return response()->json(['message' => 'Failed to create zip file.']);
    }
}
